#include <ctime>
#include <rtt/TaskContext.hpp>

struct Foo : public RTT::TaskContext
{
  Foo(std::string name) : 
    RTT::TaskContext(name, PreOperational)
  {
    addOperation("getName", &Foo::getName, this, RTT::OwnThread).doc("Get name");
    //addOperation("getName", &Foo::getName, this, RTT::ClientThread).doc("Get name");
  }

  bool configureHook(void)
  {
    fprintf(stderr, "%s\n", __PRETTY_FUNCTION__);
    m_flag_update = false;
    return true;
  }

  bool startHook(void)
  {
    fprintf(stderr, "%s : %speriodic\n", __PRETTY_FUNCTION__, getActivity()->isPeriodic()?"":"non");
    m_flag_update = true;
    return true;
  }

  void updateHook(void)
  {
    assert(m_flag_update);
    static unsigned n = 0;
    unsigned i = 0;
    timespec req;
    req.tv_sec = 0;
    req.tv_nsec = 100000000;
    n++;
    while(m_flag_update)
      {
        if((i % 64) == 0)
          {
            fprintf(stderr, "%s tick %u %u!\n", __PRETTY_FUNCTION__, n, i);
          }
        i++;
        nanosleep(&req, 0);
      }
  }

  bool breakUpdateHook(void)
  {
    fprintf(stderr, "%s\n", __PRETTY_FUNCTION__);
    m_flag_update = false;
    return true;
  }

  void stopHook(void)
  {
    assert(m_flag_update == false);
    fprintf(stderr, "%s\n", __PRETTY_FUNCTION__);
  }

  void cleanupHook(void)
  {
    fprintf(stderr, "%s\n", __PRETTY_FUNCTION__);
  }

private :

  volatile bool m_flag_update;

  std::string getName(void) const
  { 
    return "Foo"; 
  }

};

#include <ocl/Component.hpp>

// requires -DRTT_COMPONENT :
ORO_CREATE_COMPONENT( Foo );
